//! Phrase extraction components
//!
//! This module provides noun chunk detection, overlap resolution,
//! and phrase extraction with canonical form selection.

pub mod chunker;
pub mod dedup;
pub mod extraction;
